// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

class BoolQuery {
  final Op _op;
  BoolQuery(this._op);
  List<TermQuery> _termQueries = [];
  List<TermQuery> _filters = [];
  BoolQuery addQuery(TermQuery tQuery) {
    _termQueries.add(tQuery);
    return this;
  }
  BoolQuery addFilter(TermQuery tQuery) {
    _filters.add(tQuery);
    return this;
  }
  static Analyzer defaultAnalyzer =  new KeywordAnalyzer();
  BoolQuery append(String field, String text,
      {Analyzer analyzer, bool filter: false, double boost: 1.00}) {
    Analyzer ana = (analyzer == null)? defaultAnalyzer : analyzer;
    for (var token in ana.getTokens(text)) {
      if (filter) {
        addFilter(new TermQuery(new Term(field, token, boost)));
      } else {
        addQuery(new TermQuery(new Term(field, token, boost)));
      }
    }
    return this;
  }
  List<TermQuery> get termQueries => _termQueries;
  List<TermQuery> get filters => _filters;
  Op get op => _op;
}
enum Op {
  and, or
}
class TermQuery {
  final Term _term;
  TermQuery(this._term);
  Term get term => _term;
}
class Term {
  final String field;
  final String text;
  final double boost;
  Term(this.field, this.text, this.boost);
}