// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:ducene/ducene.dart';
import 'package:test/test.dart';

void main() {
  group('A group of analyzer', () {
    NGramAnalyzer uniGramAnalyzer;
    NGramAnalyzer biGramAnalyzer;
    ShingleAnalyzer wordUniGramAnalyzer;
    ShingleAnalyzer wordBiGramAnalyzer;
    WhitespaceAnalyzer whitespaceAnalyzer;
    ShingleAnalyzer biShingleAnalyzer;
    ShingleAnalyzer triShingleAnalyzer;
    EdgeNGramAnalyzer edgeNGramAnalyzer;

    setUp(() {
      uniGramAnalyzer = new NGramAnalyzer(1);
      biGramAnalyzer = new NGramAnalyzer(2);
      wordUniGramAnalyzer = new ShingleAnalyzer(uniGramAnalyzer, 2, '');
      wordBiGramAnalyzer = new ShingleAnalyzer(biGramAnalyzer, 2, '');
      whitespaceAnalyzer = new  WhitespaceAnalyzer();
      biShingleAnalyzer = new ShingleAnalyzer(whitespaceAnalyzer, 2, ' ');
      triShingleAnalyzer = new ShingleAnalyzer(whitespaceAnalyzer, 3, ' ');
      edgeNGramAnalyzer = new EdgeNGramAnalyzer(7);
    });

    test('should be 1 gram', () {
      expect(uniGramAnalyzer.getTokens('aaa bbb'),
          ['a', 'a', 'a', ' ', 'b', 'b', 'b']);
    });

    test('should be 2 gram', () {
      expect(biGramAnalyzer.getTokens('aaa bbb'),
          ['aa', 'aa', 'a ', ' b', 'bb', 'bb']);
    });

    test('should be 2 shingle with 1 gram', () {
      expect(wordUniGramAnalyzer.getTokens('aaa bbb').toList(),
          ['aa', 'aa', 'a ', ' b', 'bb', 'bb', 'b']);
    });

    test('should be 2 shingle with 2 gram', () {
      expect(wordBiGramAnalyzer.getTokens('aaa bbb').toList(),
          ['aaaa', 'aaa ', 'a  b', ' bbb', 'bbbb', 'bb']);
    });

    test('should be segments by punctuation', () {
      expect(whitespaceAnalyzer.getTokens("aa bb,cc.dd!ee?ff: gg;hh \"ii\" 'jj' \r\n kk ll"),
          ['aa', 'bb', 'cc', 'dd', 'ee', 'ff', 'gg', 'hh', 'ii', 'jj', 'kk', 'll']);
    });

    test('should be whitespace', () {
      expect(whitespaceAnalyzer.getTokens('Please divide this sentence into shingles'),
          ['please', 'divide', 'this', 'sentence', 'into', 'shingles']);
    });

    test('should be 2 shingle with whitespace', () {
      expect(biShingleAnalyzer.getTokens('Please divide this sentence into shingles'),
          ['please divide', 'divide this', 'this sentence', 'sentence into', 'into shingles', 'shingles']);
    });

    test('should be 3 shingle with whitespace', () {
      expect(triShingleAnalyzer.getTokens('Please divide this sentence into shingles'),
          ['please divide this', 'divide this sentence', 'this sentence into', 'sentence into shingles', 'into shingles', 'shingles']);
    });

    test('should be edgeNGram up to max', () {
      expect(edgeNGramAnalyzer.getTokens('12345-abcde'),
          ['1', '12', '123', '1234', '12345', '12345-', '12345-a']);
    });

    test('should be edgeNGram less text length than max', () {
      expect(edgeNGramAnalyzer.getTokens('ABC'),
          ['a', 'ab', 'abc']);
    });
  });
}

