// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'package:ducene/ducene.dart';
import 'package:test/test.dart';

void main() {
  group('A group of searcher', () {

    test('should have correct counts', () {
      IndexDirectory indexDirectory = new RAMIndexDirectory();
      IndexWriter writer = new IndexWriter(indexDirectory);
      List<Document> docs = [];
      for (int i = 0; i < 10; i++) {
        Document doc = new Document();
        doc.append("id", i.toString());
        doc.append("all", "x");
        doc.append("mv", ["m", "v"]);
        doc.append("filter", "filter");
        docs.add(doc);
      }
      writer.write(docs);

      IndexReader reader = DirectoryReader.open(indexDirectory);
      IndexSearcher searcher = new IndexSearcher(reader);

      expect(searcher.count(new BoolQuery(Op.and).append('id', '5')), 1);
      expect(searcher.count(new BoolQuery(Op.and).append('all', 'x')), 10);
      expect(searcher.count(new BoolQuery(Op.and).append('id', 'dummy')), 0);
      expect(searcher.count(new BoolQuery(Op.and).append('all', 'dummy')), 0);
      expect(searcher.count(new BoolQuery(Op.or).append('id', '5')), 1);
      expect(searcher.count(new BoolQuery(Op.or).append('all', 'x')), 10);
      expect(searcher.count(new BoolQuery(Op.or).append('id', 'dummy')), 0);
      expect(searcher.count(new BoolQuery(Op.or).append('all', 'dummy')), 0);

      expect(searcher.count(new BoolQuery(Op.or).append('mv', 'm')), 10);
      expect(searcher.count(new BoolQuery(Op.or).append('mv', 'v')), 10);

      BoolQuery q = new BoolQuery(Op.or)
          .append('all', 'x')
          .append('filter', 'filter', filter: true);
      expect(searcher.count(q), 10);
      q = new BoolQuery(Op.or)
          .append('all', 'x')
          .append('filter', 'dummy', filter: true);
      expect(searcher.count(q), 0);
      q = new BoolQuery(Op.or)
          .append('all', 'x')
          .append('filter', 'filter', filter: true)
          .append('filter', 'dummy', filter: true);
      expect(searcher.count(q), 0);
    });

  });
}

